<?php

namespace App\Http\Controllers\Admin\Base;

use App\Http\Controllers\Controller;
use App\Jobs\Base\ImportAdminJob;
use App\Repositories\Models\Base\Admin;
use App\Repositories\Models\Base\Resource;
use App\Services\Base\AdminService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Jiannei\Response\Laravel\Support\Facades\Response;

/**
 * 管理员管理
 *
 * @package namespace App\Http\Controllers;
 */
class AdminController extends Controller
{
    protected $service;

    public function __construct(AdminService $service)
    {
        parent::__construct();
        $this->service = $service;
    }

    /**
     * 列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Prettus\Repository\Exceptions\RepositoryException
     */
    public function index(Request $request)
    {
        $data = $this->service->handleList($request);
        return $this->response->success($data);
    }

    /**
     * 新增
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function store(Request $request)
    {
        $data = $this->validateData($request, [
            'name' => 'required',
            'username' => ['required', 'alpha_dash',
                function ($attribute, $value, $fail) use ($request) {
                    if (Admin::checkUsernameIsUnique($value, $request->get('id'))) {
                        $fail('账号 已存在');
                    }
                },],
            'password' => 'required|confirmed|password_level',
            'mobile' => 'required|mobile',
            'email' => 'nullable|sometimes|email',
            'sex' => 'required|in:0,1,2',
            'department_id' => 'required|integer',
            'role_ids' => 'required|array',
            'status' => 'required',
            'extra_fields' => 'nullable|sometimes|array',
        ], [
            'name' => '姓名',
            'username' => '账号',
            'sex' => '性别',
            'password' => '密码',
            'mobile' => '手机号',
            'role_ids' => '角色',
            'email' => '邮箱',
            'department_id' => '部门',
            'status' => '状态',
            'extra_fields' => '扩展字段'
        ]);

        $model = $this->service->handleStore($data);
        return $this->response->created($model, '创建成功');
    }


    /**
     * 详情
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     */
    public function show(Request $request)
    {
        $this->validate($request, ['id' => 'required|integer']);
        $model = $this->service->handleProfile($request->get('id'));
        return $this->response->success($model);
    }

    /**
     * 编辑
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function update(Request $request)
    {
        $data = $this->validateData($request, [
            'id' => 'required|integer',
            'username' => [
                'required',
                'alpha_dash',
                function ($attribute, $value, $fail) use ($request) {
                    if (Admin::checkUsernameIsUnique($value, $request->get('id'))) {
                        $fail('账号 已存在');
                    }
                },
            ],
            'name' => 'required',
            'mobile' => 'required|mobile',
            'email' => 'nullable|sometimes|email',
            'sex' => 'required|in:0,1,2',
            'department_id' => 'required|integer',
            'role_ids' => 'required|array',
            'status' => 'required|integer',
            'extra_fields' => 'nullable|sometimes|array',
        ], [
            'name' => '姓名',
            'account' => '学号',
            'headimg' => '头像',
            'sex' => '性别',
            'mobile' => '手机号',
            'email' => '邮箱',
            'grade_id' => '班级',
            'year' => '年级',
            'status' => '状态'
        ]);

        $model = $this->service->handleUpdate($data);
        return $this->response->success($model, '创建成功');
    }

    /**
     * 删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function destroy(Request $request)
    {
        $this->validate($request, ['id' => 'required|integer']);
        $re = $this->service->handleDelete($request->get('id'));
        if ($re) {
            return $this->response->ok('删除成功');
        }
        return $this->response->fail('删除失败');
    }

    /**
     * 批量删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function batchDelete(Request $request)
    {
        $ids = $this->validateData($request, ['ids' => 'required|array']);
        $re = $this->service->handleBatchDelete($ids);
        if ($re) {
            return $this->response->ok('删除成功');
        }
        return $this->response->fail('删除失败');
    }

    /**
     * 选项
     * @must
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     * @throws \Prettus\Repository\Exceptions\RepositoryException
     */
    public function selectOptions(Request $request)
    {
        $lists = $this->service->handleSelectOptions($request);
        return $this->response->success($lists);
    }

    /**
     * 用户授权
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function authorization(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
            'role_ids' => 'required|array',
        ]);

        $model = Admin::query()->find($request->get('id'));
        if (!$model) {
            return Response::fail('找不到该用户');
        }
        $role_ids = $request->get('role_ids');
        $model->syncRoles($role_ids);
        return Response::success(null);
    }

    /**
     * 重置密码
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     */
    public function changePassword(Request $request)
    {
        $data = $this->validateData($request, [
            'id' => 'required|integer',
            'password' => 'required|password_level',
        ]);

        $id = $request->get('id');

        $admin = Admin::query()->find($id);
        if (!$admin) {
            return Response::fail('找不到该用户');
        }

        $admin->password = Hash::make($data['password']);
        $admin->save();
        return Response::success([]);
    }

    /**
     * 导入
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function import(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer'
        ]);

        $resource = Resource::query()->where('id', $request->get('id'))->first();
        if (!$resource) {
            return $this->errorFail();
        }
        $this->dispatch(new ImportAdminJob($resource, login_admin_id()));
        return Response::success(null, '正在导入，请稍后');
    }
}
