<?php

namespace App\Http\Controllers\Admin\Base;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\Base\AdminTypeEnum;
use App\Repositories\Enums\Base\UserStatusEnum;
use App\Repositories\Models\Base\Admin;
use App\Repositories\Models\Base\User;
use App\Services\Base\UserService;
use Illuminate\Http\Request;

/**
 * 访客管理
 */
class UserController extends Controller
{
    /**
     * @var UserService
     */
    private $userService;

    /**
     * UserController constructor.
     *
     * @param UserService $userService
     */
    public function __construct(UserService $userService)
    {
        parent::__construct();
//        $this->middleware('checkUserPermission');

        $this->userService = $userService;
    }

    /**
     * 列表
     * @param Request $request
     *
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     * @throws \Prettus\Repository\Exceptions\RepositoryException
     */
    public function index(Request $request)
    {
        $users = $this->userService->handleList($request);
        return $this->response->success($users);
    }

    /**
     * 详情
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function show(Request $request)
    {
        $this->validate($request, ['id' => 'required|integer']);
        $user = $this->userService->handleProfile($request->get('id'));
        return $this->response->success($user);
    }

    /**
     * 删除
     * @param Request $request
     */
    public function destroy(Request $request)
    {
        $this->validate($request, ['id' => 'required|integer']);
        $re = $this->userService->handleDelete($request->get('id'));
        if ($re) {
            return $this->response->ok('删除成功');
        }
        return $this->response->fail('删除失败');
    }

    /**
     * 批量修改状态
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     * @throws \Illuminate\Validation\ValidationException
     */
    public function batchUpdateStatus(Request $request)
    {
        $data = $this->validateData($request, [
            'ids' => 'required|array',
            'status' => 'required|integer',
            'remark' => 'nullable'
        ], [
            'ids' => 'ids',
            'status' => '状态',
            'remark' => '备注'
        ]);

        User::query()->whereIn('id', $data['ids'])->update([
            'status' => $data['status'],
            'remark' => $data['remark'],
        ]);
        return $this->response->ok('操作成功');
    }

    /**
     * 更新
     * @param Request $request
     *
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function update(Request $request)
    {
        $data = $this->validateData($request, [
            'id' => 'required|integer',
            'name' => 'required',
            'mobile' => 'required|mobile',
            'sex' => 'required|in:0,1,2',
            'department_id' => 'required|integer',
            'user_no' => 'required',
            'type' => 'required',
            'grade' => 'required',
            'remark' => 'nullable',
            'class_name' => 'required',
        ], [
            'name' => '姓名',
            'mobile' => '手机号',
            'email' => '邮箱',
            'sex' => '性别',
            'department_id' => '部门',
            'user_no' => '工号',
            'type' => '类型',
            'class_name' => '班级',
            'grade' => '年级',
            'remark' => '备注',
        ]);
        $user = $this->userService->handleUpdate($data);
        return $this->response->success($user, '更新成功');
    }

    /**
     * 批量修改状态
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     * @throws \Illuminate\Validation\ValidationException
     */
    public function batchGraduate(Request $request)
    {
        $data = $this->validateData($request, [
            'year' => 'required|date_format:Y',
            'remark' => 'nullable'
        ], [
            'year' => '毕业年级',
            'remark' => '备注'
        ]);

        User::query()->where('type', AdminTypeEnum::STUDENT)->where('grade', $data['year'])->update([
            'status' => UserStatusEnum::GRADIATE,
            'remark' => $data['remark'],
        ]);
        return $this->response->ok('操作成功');
    }
}
