<?php

namespace App\Http\Controllers\Admin\Course;

use App\Http\Controllers\Controller;
use App\Services\Course\CourseService;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

/**
 * 课程管理
 *
 * @package namespace App\Http\Controllers;
 */
class CourseController extends Controller
{
    protected $service;

    public function __construct(CourseService $service)
    {
        parent::__construct();
        $this->service = $service;
    }

    /**
     * 列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Prettus\Repository\Exceptions\RepositoryException
     */
    public function index(Request $request)
    {
        $data = $this->service->handleList($request);
        return $this->response->success($data);
    }

    /**
     * 新增
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function store(Request $request)
    {
        $data = $this->validateData($request, [
            'category_id' => 'required|integer',
            'title' => 'required',
            'labels' => 'sometimes|array',
            'thumb' => 'required|integer',
            'short_description' => '',
            'description' => 'sometimes',
            'published_at' => 'sometimes|date',
            'is_rec' => 'required|integer',
            'teacher' => '',
            'settings.comment_status' => 'required|integer',
            'sort' => 'sometimes|nullable|integer',
            'status' => 'required|integer',
        ], [
            'category_id' => '课程分类',
            'title' => '课程名称',
            'labels' => '标签',
            'thumb' => '缩率图',
            'short_description' => '简介',
            'description' => '课程介绍',
            'published_at' => '发布时间',
            'is_rec' => '是否推荐',
            'teacher' => '老师',
            'settings.comment_status' => '是否开启评论',
            'sort' => '排序',
            'status' => '状态',
        ]);

        $data['admin_id'] = login_admin_id();
        $data['slug'] = Str::random();

        $model = $this->service->handleStore($data);
        return $this->response->created($model, '创建成功');
    }


    /**
     * 详情
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     */
    public function show(Request $request)
    {
        $this->validate($request, ['id' => 'required|integer']);
        $model = $this->service->handleProfile($request->get('id'));
        return $this->response->success($model);
    }

    /**
     * 编辑
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function update(Request $request)
    {
        $data = $this->validateData($request, [
            'id' => 'required|integer',
            'category_id' => 'required|integer',
            'title' => 'required',
            'labels' => 'sometimes|array',
            'thumb' => 'required|integer',
            'short_description' => '',
            'description' => 'sometimes',
            'published_at' => 'sometimes|date',
            'is_rec' => 'required|integer',
            'teacher' => '',
            'settings.comment_status' => 'required|integer',
            'sort' => 'sometimes|nullable|integer',
            'status' => 'required|integer',
        ], [
            'category_id' => '课程分类',
            'title' => '课程名称',
            'labels' => '标签',
            'thumb' => '缩率图',
            'short_description' => '简介',
            'description' => '课程介绍',
            'published_at' => '发布时间',
            'is_rec' => '是否推荐',
            'teacher' => '老师',
            'settings.comment_status' => '是否开启评论',
            'sort' => '排序',
            'status' => '状态',
        ]);

        $model = $this->service->handleUpdate($data);
        return $this->response->success($model, '创建成功');
    }

    /**
     * 删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function destroy(Request $request)
    {
        $this->validate($request, ['id' => 'required|integer']);
        $re = $this->service->handleDelete($request->get('id'));
        if ($re) {
            return $this->response->ok('删除成功');
        }
        return $this->response->fail('删除失败');
    }

    /**
     * 批量删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function batchDelete(Request $request)
    {
        $ids = $this->validateData($request, ['ids' => 'required|array']);
        $re = $this->service->handleBatchDelete($ids);
        if ($re) {
            return $this->response->ok('删除成功');
        }
        return $this->response->fail('删除失败');
    }

    /**
     * 选项
     * @must
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     * @throws \Prettus\Repository\Exceptions\RepositoryException
     */
    public function selectOptions(Request $request)
    {
        $lists = $this->service->handleSelectOptions($request);
        return $this->response->success($lists);
    }

}
