<?php

namespace App\Http\Controllers\Admin\Exam;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\PermissionEnum;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Exam\Paper;
use App\Repositories\Models\Exam\Topic;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\Exam\PaperRepository;
use App\Repositories\Validators\Exam\PaperValidator;

/**
 * Class PapersController.
 *
 * @package namespace App\Http\Controllers\Exam;
 */
class PapersController extends Controller
{
    /**
     * @var PaperRepository
     */
    protected $repository;

    /**
     * @var PaperValidator
     */
    protected $validator;

    /**
     * PapersController constructor.
     *
     * @param PaperRepository $repository
     * @param PaperValidator $validator
     */
    public function __construct(PaperRepository $repository, PaperValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * 列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index(Request $request)
    {
        $lists = $this->repository->paginate($request->get('per_page', self::PAGE_NUM));
        return Response::success($lists);
    }

    /**
     * 添加
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function store(Request $request)
    {
        $this->isCan(PermissionEnum::controller_paper_add);
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));

        $this->validate($request, [
            'topics' => 'required|array',
        ]);

        $topics = $request->get('topics', []);

        if (count($topics)) {
            $nums = Topic::query()->whereIn('id', $topics)->where('status', ModelStatusEnum::OK)->count();
            if ($nums !== count($topics)) {
                return $this->error('提交试题有问题');
            }
        }

        try {
            $data['admin_id'] = login_admin_id();
            $data['topic_nums'] = $nums;
            unset($data['topics']);
            $model = Paper::query()->create($data);
            foreach ($topics as $k => $topic) {
                $index = $k + 1;
                $topics[$topic] = ['sort' => $index];
            }
            $model->topics()->sync($topics);
            return Response::success($model);
        } catch (\Exception $e) {
            return $this->error($e);
        }
    }

    /**
     * 详情
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show($id)
    {
        $data = $this->repository->find($id);
        return Response::success($data);
    }

    /**
     * 更新
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function update(Request $request, $id)
    {
        $this->isCan(PermissionEnum::controller_paper_add);
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        $this->validate($request, [
            'topics' => 'required|array',
        ]);

        $topics = $request->get('topics', []);

        if (count($topics)) {
            $nums = Topic::query()->whereIn('id', $topics)->where('status', ModelStatusEnum::OK)->count();
            if ($nums !== count($topics)) {
                return $this->error('提交试题有问题');
            }
        }
        $model = Paper::query()->find($id);
        if (!$model) {
            return $this->error('找不到该试卷');
        }
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
            $data['topic_nums'] = $nums;
            $model->fill($data);
            $model->save();
            foreach ($topics as $k => $topic) {
                $index = $k + 1;
                $topics[$topic] = ['sort' => $index];
            }
            $model->topics()->sync($topics);
            return Response::success($model);
        } catch (\Exception $e) {
            $this->error($e);
        }
    }

    /**
     * 删除
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function destroy($id)
    {
        $this->isCan(PermissionEnum::controller_paper_del);
        try {
            $this->repository->skipPresenter()->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 批量删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function delete(Request $request)
    {
        $this->isCan(PermissionEnum::controller_paper_del);
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 选项
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function selectOptions(Request $request)
    {
        $data = $this->repository->skipPresenter()->all(['name', 'room_no', 'id']);
        return Response::success($data);
    }

    /**
     * 暂停
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     */
    public function pause(Request $request)
    {
        $this->isCan(PermissionEnum::controller_paper_del);
        $this->validate($request, [
            'id' => 'required|integer',
            'type' => 'sometimes|nullable|integer',
        ]);
        $status = ModelStatusEnum::PAUSE;
        if ($request->get('type')) {
            $status = ModelStatusEnum::OK;
        }

        $re = Paper::query()->where('id', $request->get('id'))->update(['status' => $status]);
        if ($re) {
            return Response::success([]);
        }
        return $this->error('操作失败');
    }
}
