<?php

/**
 * 登录限制
 * mead
 */

namespace App\Support\Traits;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;

trait LoginLimit
{
    /**
     * The maximum number of attempts to allow.
     *
     * @var int
     */
    protected $maxAttempts = 5;

    /**
     * The number of minutes to throttle for.
     *
     * @var int|float|int[]|float[]
     */
    protected $decayMinutes = [5, 10, 30];

    /**
     * 登录失败记录
     * @param Request $request
     * @param bool $maxAttempts
     * @param bool $decayMinutes
     * @return bool
     * Author: Mead
     */
    public function storeLoginLog(Request $request, $username = '', $guard = 'admins', $maxAttempts = false, $decayMinutes = false)
    {
        if (!$maxAttempts) {
            $maxAttempts = $this->maxAttempts;
        }

        if (!$decayMinutes) {
            $decayMinutes = $this->decayMinutes;
        }

        $key = $this->throttleKey($request) . ":{$guard}" . ":{$username}";

        if (!Cache::has($key . ':step')) {
            Cache::add($key . ':step', 1, Carbon::parse(date('Y-m-d'))->addDays(1));
        } else {
            Cache::increment($key . ':step');
        }

        $nums = Cache::get($key . ':step', 1);

        if (!($nums % $maxAttempts)) {
            $step = $nums / $maxAttempts;
            $index = $step <= count($decayMinutes) ? ($step - 1) : false;
            if ($index === false) {
                $minutes = false;
            } else {
                $minutes = $decayMinutes[$index];
            }

            $time = str2arr(Cache::get($key . ':timer', ''));
            Cache::put($key . ':timer', php2js([
                'time' => $minutes ? Carbon::now()->addMinutes($minutes)->toDateTimeString() : false,
                'nums' => array_key_exists('time', $time) ? $time['time'] + 1 : 1
            ]), $minutes ? Carbon::now()->addMinutes($minutes) : Carbon::parse(date('Y-m-d'))->addDays(1));
        }
        return true;
    }

    /**
     * 获取key
     */
    public function throttleKey(Request $request)
    {
        return sha1(implode('|', array_merge(
            [$request->getMethod()],
            [$request->getHost(), $request->getUri(), $request->getClientIp()]
        )));
    }

    /**
     * 是否可以登录
     * @param Request $request
     * @return bool|string
     * Author: Mead
     */
    public function isCanLogin(Request $request, $username = '', $guard = 'admins')
    {
        $key = $this->throttleKey($request) . ":{$guard}" . ":{$username}";
        $msg = false;
        if (Cache::has($key . ':timer')) {
            $msg = '今天尝试次数太多，请明天再试。';
            $re = js2php(Cache::get($key . ':timer', ''));

            if (array_key_exists('time', $re)) {
                if ($re['time'] != false) {
                    $minutes = Carbon::parse($re['time'])->diffInMinutes(Carbon::now()) + 1;
                    $msg = "尝试次数太多，请{$minutes}分钟后再试。";
                }
            }
        }
        return $msg;
    }

    /**
     * 清楚缓存
     * @param Request $request
     * Author: Mead
     */
    public function clearLoginLogs(Request $request, $username = '', $guard = 'admins')
    {
        $key = $this->throttleKey($request) . ":{$guard}" . ":{$username}";
        Cache::forget($key . ':step');
    }


    /**
     * 检查登录设备是否过期
     * @param $token
     * @param $user_id
     * @param $guard
     * @return bool
     */
    public function singleLoginCheck($token, $user_id, $guard)
    {
        $isSingleLogin = config('site.setting.isSingleLogin', true);
        if (!$isSingleLogin) return true;
        $redis_token = Cache::tags('login')->get("SingleLoginLimit:{$guard}:{$user_id}", false);
        if ($redis_token && (string)$redis_token === (string)$token) {
            return true;
        }
        return false;
    }

    /**
     * 设置登录token
     * @param $user_id
     * @param $token
     * @param $guard
     * @return bool
     */
    private function singleLoginSetToken($user_id, $token, $guard)
    {
        return Cache::tags('login')->put("SingleLoginLimit:{$guard}:{$user_id}", $token);
    }
}
